@@ -22,6 +22,8 @@
 #include <config.h>
 #endif
 
+#include "gedit-app.h"
+
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -35,8 +37,7 @@
 #include <girepository.h>
 #endif
 
-#include "gedit-app.h"
-#include "gedit-commands.h"
+#include "gedit-commands-private.h"
 #include "gedit-notebook.h"
 #include "gedit-debug.h"
 #include "gedit-utils.h"
@@ -289,354 +290,6 @@ gedit_app_set_window_title_impl (GeditApp    *app,
 	gtk_window_set_title (GTK_WINDOW (window), title);
 }
 
-static void
-new_window_activated (GSimpleAction *action,
-                      GVariant      *parameter,
-                      gpointer       user_data)
-{
-	GeditApp *app;
-	GeditWindow *window;
-
-	app = GEDIT_APP (user_data);
-	window = gedit_app_create_window (app, NULL);
-
-	gedit_debug_message (DEBUG_APP, "Show window");
-	gtk_widget_show (GTK_WIDGET (window));
-
-	gedit_debug_message (DEBUG_APP, "Create tab");
-	gedit_window_create_tab (window, TRUE);
-
-	gtk_window_present (GTK_WINDOW (window));
-}
-
-static void
-preferences_activated (GSimpleAction  *action,
-                       GVariant       *parameter,
-                       gpointer        user_data)
-{
-	GtkApplication *app;
-	GeditWindow *window;
-
-	app = GTK_APPLICATION (user_data);
-	window = GEDIT_WINDOW (gtk_application_get_active_window (app));
-
-	gedit_show_preferences_dialog (window);
-}
-
-static void
-help_activated (GSimpleAction *action,
-                GVariant      *parameter,
-                gpointer       user_data)
-{
-	GtkApplication *app;
-	GeditWindow *window;
-
-	app = GTK_APPLICATION (user_data);
-	window = GEDIT_WINDOW (gtk_application_get_active_window (app));
-
-	_gedit_cmd_help_contents (NULL, window);
-}
-
-static void
-about_activated (GSimpleAction  *action,
-                 GVariant       *parameter,
-                 gpointer        user_data)
-{
-	GtkApplication *app;
-	GeditWindow *window;
-
-	app = GTK_APPLICATION (user_data);
-	window = GEDIT_WINDOW (gtk_application_get_active_window (app));
-
-	_gedit_cmd_help_about (NULL, window);
-}
-
-static void
-quit_activated (GSimpleAction *action,
-                GVariant      *parameter,
-                gpointer       user_data)
-{
-	_gedit_cmd_file_quit (NULL, NULL, NULL);
-}
-
-static GActionEntry app_entries[] = {
-	{ "new-window", new_window_activated, NULL, NULL, NULL },
-	{ "preferences", preferences_activated, NULL, NULL, NULL },
-	{ "help", help_activated, NULL, NULL, NULL },
-	{ "about", about_activated, NULL, NULL, NULL },
-	{ "quit", quit_activated, NULL, NULL, NULL }
-};
-
-static void
-extension_added (PeasExtensionSet *extensions,
-		 PeasPluginInfo   *info,
-		 PeasExtension    *exten,
-		 GeditApp         *app)
-{
-	gedit_app_activatable_activate (GEDIT_APP_ACTIVATABLE (exten));
-}
-
-static void
-extension_removed (PeasExtensionSet *extensions,
-		   PeasPluginInfo   *info,
-		   PeasExtension    *exten,
-		   GeditApp         *app)
-{
-	gedit_app_activatable_deactivate (GEDIT_APP_ACTIVATABLE (exten));
-}
-
-static void
-load_accels (void)
-{
-	gchar *filename;
-
-	filename = g_build_filename (gedit_dirs_get_user_config_dir (),
-				     "accels",
-				     NULL);
-	if (filename != NULL)
-	{
-		gedit_debug_message (DEBUG_APP, "Loading keybindings from %s\n", filename);
-		gtk_accel_map_load (filename);
-		g_free (filename);
-	}
-}
-
-static void
-theme_changed (GtkSettings *settings,
-	       GParamSpec  *pspec,
-	       gpointer     data)
-{
-	static GtkCssProvider *provider;
-
-	if (pspec == NULL || g_str_equal (pspec->name, "gtk-theme-name"))
-	{
-		gchar *theme;
-		GdkScreen *screen;
-
-		g_object_get (settings, "gtk-theme-name", &theme, NULL);
-		screen = gdk_screen_get_default ();
-
-		if (g_str_equal (theme, "Adwaita"))
-		{
-			if (provider == NULL)
-			{
-				GFile *file;
-
-				provider = gtk_css_provider_new ();
-				file = g_file_new_for_uri ("resource:///org/gnome/gedit/css/gedit.adwaita.css");
-				gtk_css_provider_load_from_file (provider, file, NULL);
-				g_object_unref (file);
-			}
-
-			gtk_style_context_add_provider_for_screen (screen,
-								   GTK_STYLE_PROVIDER (provider),
-								   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-		}
-		else if (provider != NULL)
-		{
-			gtk_style_context_remove_provider_for_screen (screen,
-								      GTK_STYLE_PROVIDER (provider));
-			g_clear_object (&provider);
-		}
-
-		g_free (theme);
-	}
-}
-
-static void
-setup_theme_extensions (void)
-{
-	GtkSettings *settings;
-
-	settings = gtk_settings_get_default ();
-	g_signal_connect (settings, "notify", G_CALLBACK (theme_changed), NULL);
-	theme_changed (settings, NULL, NULL);
-}
-
-static GMenuModel *
-get_menu_model (GeditApp   *app,
-                const char *id)
-{
-	GMenu *menu;
-
-	menu = gtk_application_get_menu_by_id (GTK_APPLICATION (app), id);
-
-	return menu ? G_MENU_MODEL (g_object_ref_sink (menu)) : NULL;
-}
-
-static void
-add_accelerator (GeditApp    *app,
-                 const gchar *action_name,
-                 const gchar *accel)
-{
-	const gchar *vaccels[] = {
-		accel,
-		NULL
-	};
-
-	gtk_application_set_accels_for_action (GTK_APPLICATION (app), action_name, vaccels);
-}
-
-static void
-load_css_from_resource (const gchar *filename,
-                        gboolean     required)
-{
-	GError *error = NULL;
-	GFile *css_file;
-	GtkCssProvider *provider;
-	gchar *resource_name;
-
-	resource_name = g_strdup_printf ("resource:///org/gnome/gedit/css/%s", filename);
-	css_file = g_file_new_for_uri (resource_name);
-	g_free (resource_name);
-
-	if (!required && !g_file_query_exists (css_file, NULL))
-	{
-		return;
-	}
-
-	provider = gtk_css_provider_new ();
-
-	if (gtk_css_provider_load_from_file (provider, css_file, &error))
-	{
-		gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-		                                           GTK_STYLE_PROVIDER (provider),
-		                                           GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-		g_object_unref (provider);
-	}
-	else
-	{
-		g_warning ("Could not load css provider: %s", error->message);
-		g_error_free (error);
-	}
-
-	g_object_unref (css_file);
-}
-
-static void
-gedit_app_startup (GApplication *application)
-{
-	GeditApp *app = GEDIT_APP (application);
-	GtkSourceStyleSchemeManager *manager;
-	const gchar *dir;
-	gchar *icon_dir;
-
-	G_APPLICATION_CLASS (gedit_app_parent_class)->startup (application);
-
-	/* Setup debugging */
-	gedit_debug_init ();
-	gedit_debug_message (DEBUG_APP, "Startup");
-
-	gedit_debug_message (DEBUG_APP, "Set icon");
-
-	dir = gedit_dirs_get_gedit_data_dir ();
-	icon_dir = g_build_filename (dir, "icons", NULL);
-
-	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (), icon_dir);
-	g_free (icon_dir);
-
-	setup_theme_extensions ();
-
-#ifndef ENABLE_GVFS_METADATA
-	gedit_metadata_manager_init ();
-#endif
-
-	/* Load settings */
-	app->priv->settings = gedit_settings_new ();
-	app->priv->ui_settings = g_settings_new ("org.gnome.gedit.preferences.ui");
-	app->priv->window_settings = g_settings_new ("org.gnome.gedit.state.window");
-
-	/* initial lockdown state */
-	app->priv->lockdown = gedit_settings_get_lockdown (GEDIT_SETTINGS (app->priv->settings));
-
-	g_action_map_add_action_entries (G_ACTION_MAP (app),
-	                                 app_entries,
-	                                 G_N_ELEMENTS (app_entries),
-	                                 app);
-
-	/* menus */
-	app->priv->window_menu = gtk_application_get_menubar (GTK_APPLICATION (app));
-
-	if (app->priv->window_menu == NULL)
-	{
-		app->priv->window_menu = get_menu_model (app, "gear-menu");
-	}
-	else
-	{
-		g_object_ref (app->priv->window_menu);
-	}
-
-	app->priv->notebook_menu = get_menu_model (app, "notebook-menu");
-	app->priv->tab_width_menu = get_menu_model (app, "tab-width-menu");
-	app->priv->line_col_menu = get_menu_model (app, "line-col-menu");
-
-	/* Accelerators */
-	add_accelerator (app, "app.new-window", "<Primary>N");
-	add_accelerator (app, "app.quit", "<Primary>Q");
-	add_accelerator (app, "app.help", "F1");
-
-	add_accelerator (app, "win.gear-menu", "F10");
-	add_accelerator (app, "win.open", "<Primary>O");
-	add_accelerator (app, "win.save", "<Primary>S");
-	add_accelerator (app, "win.save-as", "<Primary><Shift>S");
-	add_accelerator (app, "win.save-all", "<Primary><Shift>L");
-	add_accelerator (app, "win.new-tab", "<Primary>T");
-	add_accelerator (app, "win.reopen-closed-tab", "<Primary><Shift>T");
-	add_accelerator (app, "win.close", "<Primary>W");
-	add_accelerator (app, "win.close-all", "<Primary><Shift>W");
-	add_accelerator (app, "win.print", "<Primary>P");
-	add_accelerator (app, "win.find", "<Primary>F");
-	add_accelerator (app, "win.find-next", "<Primary>G");
-	add_accelerator (app, "win.find-prev", "<Primary><Shift>G");
-	add_accelerator (app, "win.replace", "<Primary>H");
-	add_accelerator (app, "win.clear-highlight", "<Primary><Shift>K");
-	add_accelerator (app, "win.goto-line", "<Primary>I");
-	add_accelerator (app, "win.focus-active-view", "Escape");
-	add_accelerator (app, "win.side-panel", "F9");
-	add_accelerator (app, "win.bottom-panel", "<Primary>F9");
-	add_accelerator (app, "win.fullscreen", "F11");
-	add_accelerator (app, "win.new-tab-group", "<Primary><Alt>N");
-	add_accelerator (app, "win.previous-tab-group", "<Primary><Shift><Alt>Page_Up");
-	add_accelerator (app, "win.next-tab-group", "<Primary><Shift><Alt>Page_Down");
-	add_accelerator (app, "win.previous-document", "<Primary><Alt>Page_Up");
-	add_accelerator (app, "win.next-document", "<Primary><Alt>Page_Down");
-
-	load_accels ();
-
-	/* Load custom css */
-	load_css_from_resource ("gedit-style.css", TRUE);
-	load_css_from_resource ("gedit-style-os.css", FALSE);
-
-	/*
-	 * We use the default gtksourceview style scheme manager so that plugins
-	 * can obtain it easily without a gedit specific api, but we need to
-	 * add our search path at startup before the manager is actually used.
-	 */
-	manager = gtk_source_style_scheme_manager_get_default ();
-	gtk_source_style_scheme_manager_append_search_path (manager,
-	                                                    gedit_dirs_get_user_styles_dir ());
-
-	app->priv->engine = gedit_plugins_engine_get_default ();
-	app->priv->extensions = peas_extension_set_new (PEAS_ENGINE (app->priv->engine),
-	                                                GEDIT_TYPE_APP_ACTIVATABLE,
-	                                                "app", app,
-	                                                NULL);
-
-	g_signal_connect (app->priv->extensions,
-	                  "extension-added",
-	                  G_CALLBACK (extension_added),
-	                  app);
-
-	g_signal_connect (app->priv->extensions,
-	                  "extension-removed",
-	                  G_CALLBACK (extension_removed),
-	                  app);
-
-	peas_extension_set_foreach (app->priv->extensions,
-	                            (PeasExtensionSetForeachFunc) extension_added,
-	                            app);
-}
-
 static gboolean
 is_in_viewport (GtkWindow    *window,
 		GdkScreen    *screen,
@@ -824,6 +477,375 @@ open_files (GApplication            *application,
 	gtk_window_present (GTK_WINDOW (window));
 }
 
+static void
+new_window_activated (GSimpleAction *action,
+                      GVariant      *parameter,
+                      gpointer       user_data)
+{
+	GeditApp *app;
+	GeditWindow *window;
+
+	app = GEDIT_APP (user_data);
+	window = gedit_app_create_window (app, NULL);
+
+	gedit_debug_message (DEBUG_APP, "Show window");
+	gtk_widget_show (GTK_WIDGET (window));
+
+	gedit_debug_message (DEBUG_APP, "Create tab");
+	gedit_window_create_tab (window, TRUE);
+
+	gtk_window_present (GTK_WINDOW (window));
+}
+
+static void
+new_document_activated (GSimpleAction *action,
+                        GVariant      *parameter,
+                        gpointer       user_data)
+{
+	GApplication *application = G_APPLICATION (user_data);
+
+	open_files (application,
+	            FALSE,
+	            TRUE,
+	            NULL,
+	            0,
+	            0,
+	            NULL,
+	            NULL,
+	            NULL,
+	            NULL);
+}
+
+static void
+preferences_activated (GSimpleAction  *action,
+                       GVariant       *parameter,
+                       gpointer        user_data)
+{
+	GtkApplication *app;
+	GeditWindow *window;
+
+	app = GTK_APPLICATION (user_data);
+	window = GEDIT_WINDOW (gtk_application_get_active_window (app));
+
+	gedit_show_preferences_dialog (window);
+}
+
+static void
+help_activated (GSimpleAction *action,
+                GVariant      *parameter,
+                gpointer       user_data)
+{
+	GtkApplication *app;
+	GeditWindow *window;
+
+	app = GTK_APPLICATION (user_data);
+	window = GEDIT_WINDOW (gtk_application_get_active_window (app));
+
+	_gedit_cmd_help_contents (NULL, window);
+}
+
+static void
+about_activated (GSimpleAction  *action,
+                 GVariant       *parameter,
+                 gpointer        user_data)
+{
+	GtkApplication *app;
+	GeditWindow *window;
+
+	app = GTK_APPLICATION (user_data);
+	window = GEDIT_WINDOW (gtk_application_get_active_window (app));
+
+	_gedit_cmd_help_about (NULL, window);
+}
+
+static void
+quit_activated (GSimpleAction *action,
+                GVariant      *parameter,
+                gpointer       user_data)
+{
+	_gedit_cmd_file_quit (NULL, NULL, NULL);
+}
+
+static GActionEntry app_entries[] = {
+	{ "new-window", new_window_activated, NULL, NULL, NULL },
+	{ "new-document", new_document_activated, NULL, NULL, NULL },
+	{ "preferences", preferences_activated, NULL, NULL, NULL },
+	{ "help", help_activated, NULL, NULL, NULL },
+	{ "about", about_activated, NULL, NULL, NULL },
+	{ "quit", quit_activated, NULL, NULL, NULL }
+};
+
+static void
+extension_added (PeasExtensionSet *extensions,
+		 PeasPluginInfo   *info,
+		 PeasExtension    *exten,
+		 GeditApp         *app)
+{
+	gedit_app_activatable_activate (GEDIT_APP_ACTIVATABLE (exten));
+}
+
+static void
+extension_removed (PeasExtensionSet *extensions,
+		   PeasPluginInfo   *info,
+		   PeasExtension    *exten,
+		   GeditApp         *app)
+{
+	gedit_app_activatable_deactivate (GEDIT_APP_ACTIVATABLE (exten));
+}
+
+static void
+load_accels (void)
+{
+	gchar *filename;
+
+	filename = g_build_filename (gedit_dirs_get_user_config_dir (),
+				     "accels",
+				     NULL);
+	if (filename != NULL)
+	{
+		gedit_debug_message (DEBUG_APP, "Loading keybindings from %s\n", filename);
+		gtk_accel_map_load (filename);
+		g_free (filename);
+	}
+}
+
+static void
+theme_changed (GtkSettings *settings,
+	       GParamSpec  *pspec,
+	       gpointer     data)
+{
+	static GtkCssProvider *provider;
+
+	if (pspec == NULL || g_str_equal (pspec->name, "gtk-theme-name"))
+	{
+		gchar *theme;
+		GdkScreen *screen;
+
+		g_object_get (settings, "gtk-theme-name", &theme, NULL);
+		screen = gdk_screen_get_default ();
+
+		if (g_str_equal (theme, "Adwaita"))
+		{
+			if (provider == NULL)
+			{
+				GFile *file;
+
+				provider = gtk_css_provider_new ();
+				file = g_file_new_for_uri ("resource:///org/gnome/gedit/css/gedit.adwaita.css");
+				gtk_css_provider_load_from_file (provider, file, NULL);
+				g_object_unref (file);
+			}
+
+			gtk_style_context_add_provider_for_screen (screen,
+								   GTK_STYLE_PROVIDER (provider),
+								   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+		}
+		else if (provider != NULL)
+		{
+			gtk_style_context_remove_provider_for_screen (screen,
+								      GTK_STYLE_PROVIDER (provider));
+			g_clear_object (&provider);
+		}
+
+		g_free (theme);
+	}
+}
+
+static void
+setup_theme_extensions (void)
+{
+	GtkSettings *settings;
+
+	settings = gtk_settings_get_default ();
+	g_signal_connect (settings, "notify", G_CALLBACK (theme_changed), NULL);
+	theme_changed (settings, NULL, NULL);
+}
+
+static GMenuModel *
+get_menu_model (GeditApp   *app,
+                const char *id)
+{
+	GMenu *menu;
+
+	menu = gtk_application_get_menu_by_id (GTK_APPLICATION (app), id);
+
+	return menu ? G_MENU_MODEL (g_object_ref_sink (menu)) : NULL;
+}
+
+static void
+add_accelerator (GeditApp    *app,
+                 const gchar *action_name,
+                 const gchar *accel)
+{
+	const gchar *vaccels[] = {
+		accel,
+		NULL
+	};
+
+	gtk_application_set_accels_for_action (GTK_APPLICATION (app), action_name, vaccels);
+}
+
+static void
+load_css_from_resource (const gchar *filename,
+                        gboolean     required)
+{
+	GError *error = NULL;
+	GFile *css_file;
+	GtkCssProvider *provider;
+	gchar *resource_name;
+
+	resource_name = g_strdup_printf ("resource:///org/gnome/gedit/css/%s", filename);
+	css_file = g_file_new_for_uri (resource_name);
+	g_free (resource_name);
+
+	if (!required && !g_file_query_exists (css_file, NULL))
+	{
+		g_object_unref (css_file);
+		return;
+	}
+
+	provider = gtk_css_provider_new ();
+
+	if (gtk_css_provider_load_from_file (provider, css_file, &error))
+	{
+		gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
+		                                           GTK_STYLE_PROVIDER (provider),
+		                                           GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+		g_object_unref (provider);
+	}
+	else
+	{
+		g_warning ("Could not load css provider: %s", error->message);
+		g_error_free (error);
+	}
+
+	g_object_unref (css_file);
+}
+
+static void
+gedit_app_startup (GApplication *application)
+{
+	GeditApp *app = GEDIT_APP (application);
+	GtkSourceStyleSchemeManager *manager;
+	const gchar *dir;
+	gchar *icon_dir;
+
+	G_APPLICATION_CLASS (gedit_app_parent_class)->startup (application);
+
+	/* Setup debugging */
+	gedit_debug_init ();
+	gedit_debug_message (DEBUG_APP, "Startup");
+
+	gedit_debug_message (DEBUG_APP, "Set icon");
+
+	dir = gedit_dirs_get_gedit_data_dir ();
+	icon_dir = g_build_filename (dir, "icons", NULL);
+
+	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (), icon_dir);
+	g_free (icon_dir);
+
+	setup_theme_extensions ();
+
+#ifndef ENABLE_GVFS_METADATA
+	gedit_metadata_manager_init ();
+#endif
+
+	/* Load settings */
+	app->priv->settings = gedit_settings_new ();
+	app->priv->ui_settings = g_settings_new ("org.gnome.gedit.preferences.ui");
+	app->priv->window_settings = g_settings_new ("org.gnome.gedit.state.window");
+
+	/* initial lockdown state */
+	app->priv->lockdown = gedit_settings_get_lockdown (GEDIT_SETTINGS (app->priv->settings));
+
+	g_action_map_add_action_entries (G_ACTION_MAP (app),
+	                                 app_entries,
+	                                 G_N_ELEMENTS (app_entries),
+	                                 app);
+
+	/* menus */
+	app->priv->window_menu = gtk_application_get_menubar (GTK_APPLICATION (app));
+
+	if (app->priv->window_menu == NULL)
+	{
+		app->priv->window_menu = get_menu_model (app, "gear-menu");
+	}
+	else
+	{
+		g_object_ref (app->priv->window_menu);
+	}
+
+	app->priv->notebook_menu = get_menu_model (app, "notebook-menu");
+	app->priv->tab_width_menu = get_menu_model (app, "tab-width-menu");
+	app->priv->line_col_menu = get_menu_model (app, "line-col-menu");
+
+	/* Accelerators */
+	add_accelerator (app, "app.new-window", "<Primary>N");
+	add_accelerator (app, "app.quit", "<Primary>Q");
+	add_accelerator (app, "app.help", "F1");
+
+	add_accelerator (app, "win.gear-menu", "F10");
+	add_accelerator (app, "win.open", "<Primary>O");
+	add_accelerator (app, "win.save", "<Primary>S");
+	add_accelerator (app, "win.save-as", "<Primary><Shift>S");
+	add_accelerator (app, "win.save-all", "<Primary><Shift>L");
+	add_accelerator (app, "win.new-tab", "<Primary>T");
+	add_accelerator (app, "win.reopen-closed-tab", "<Primary><Shift>T");
+	add_accelerator (app, "win.close", "<Primary>W");
+	add_accelerator (app, "win.close-all", "<Primary><Shift>W");
+	add_accelerator (app, "win.print", "<Primary>P");
+	add_accelerator (app, "win.find", "<Primary>F");
+	add_accelerator (app, "win.find-next", "<Primary>G");
+	add_accelerator (app, "win.find-prev", "<Primary><Shift>G");
+	add_accelerator (app, "win.replace", "<Primary>H");
+	add_accelerator (app, "win.clear-highlight", "<Primary><Shift>K");
+	add_accelerator (app, "win.goto-line", "<Primary>I");
+	add_accelerator (app, "win.focus-active-view", "Escape");
+	add_accelerator (app, "win.side-panel", "F9");
+	add_accelerator (app, "win.bottom-panel", "<Primary>F9");
+	add_accelerator (app, "win.fullscreen", "F11");
+	add_accelerator (app, "win.new-tab-group", "<Primary><Alt>N");
+	add_accelerator (app, "win.previous-tab-group", "<Primary><Shift><Alt>Page_Up");
+	add_accelerator (app, "win.next-tab-group", "<Primary><Shift><Alt>Page_Down");
+	add_accelerator (app, "win.previous-document", "<Primary><Alt>Page_Up");
+	add_accelerator (app, "win.next-document", "<Primary><Alt>Page_Down");
+
+	load_accels ();
+
+	/* Load custom css */
+	load_css_from_resource ("gedit-style.css", TRUE);
+	load_css_from_resource ("gedit-style-os.css", FALSE);
+
+	/*
+	 * We use the default gtksourceview style scheme manager so that plugins
+	 * can obtain it easily without a gedit specific api, but we need to
+	 * add our search path at startup before the manager is actually used.
+	 */
+	manager = gtk_source_style_scheme_manager_get_default ();
+	gtk_source_style_scheme_manager_append_search_path (manager,
+	                                                    gedit_dirs_get_user_styles_dir ());
+
+	app->priv->engine = gedit_plugins_engine_get_default ();
+	app->priv->extensions = peas_extension_set_new (PEAS_ENGINE (app->priv->engine),
+	                                                GEDIT_TYPE_APP_ACTIVATABLE,
+	                                                "app", app,
+	                                                NULL);
+
+	g_signal_connect (app->priv->extensions,
+	                  "extension-added",
+	                  G_CALLBACK (extension_added),
+	                  app);
+
+	g_signal_connect (app->priv->extensions,
+	                  "extension-removed",
+	                  G_CALLBACK (extension_removed),
+	                  app);
+
+	peas_extension_set_foreach (app->priv->extensions,
+	                            (PeasExtensionSetForeachFunc) extension_added,
+	                            app);
+}
+
 static void
 gedit_app_activate (GApplication *application)
 {
