@@ -451,7 +451,7 @@ gedit_document_class_init (GeditDocumentClass *klass)
 	document_signals[LOADED] =
 		g_signal_new ("loaded",
 			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
+			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GeditDocumentClass, loaded),
 			      NULL, NULL, NULL,
 			      G_TYPE_NONE, 0);
@@ -491,7 +491,7 @@ gedit_document_class_init (GeditDocumentClass *klass)
 	document_signals[SAVED] =
 		g_signal_new ("saved",
 			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
+			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GeditDocumentClass, saved),
 			      NULL, NULL, NULL,
 			      G_TYPE_NONE, 0);
@@ -1619,21 +1619,6 @@ gedit_document_get_metadata (GeditDocument *doc,
 	return NULL;
 }
 
-static void
-set_attributes_cb (GFile        *location,
-		   GAsyncResult *result)
-{
-	GError *error = NULL;
-
-	g_file_set_attributes_finish (location, result, NULL, &error);
-
-	if (error != NULL)
-	{
-		g_warning ("Set document metadata failed: %s", error->message);
-		g_error_free (error);
-	}
-}
-
 /**
  * gedit_document_set_metadata:
  * @doc: a #GeditDocument
@@ -1668,32 +1653,55 @@ gedit_document_set_metadata (GeditDocument *doc,
 		if (value != NULL)
 		{
 			g_file_info_set_attribute_string (info, key, value);
+
+			if (doc->priv->metadata_info != NULL)
+			{
+				g_file_info_set_attribute_string (doc->priv->metadata_info, key, value);
+			}
 		}
 		else
 		{
 			/* Unset the key */
-			g_file_info_remove_attribute (info, key);
+			g_file_info_set_attribute (info,
+						   key,
+						   G_FILE_ATTRIBUTE_TYPE_INVALID,
+						   NULL);
+
+			if (doc->priv->metadata_info != NULL)
+			{
+				g_file_info_set_attribute (doc->priv->metadata_info,
+							   key,
+							   G_FILE_ATTRIBUTE_TYPE_INVALID,
+							   NULL);
+			}
 		}
 	}
 
 	va_end (var_args);
 
-	if (doc->priv->metadata_info != NULL)
-	{
-		g_file_info_copy_into (info, doc->priv->metadata_info);
-	}
-
 	location = gtk_source_file_get_location (doc->priv->file);
 
 	if (location != NULL)
 	{
-		g_file_set_attributes_async (location,
-					     info,
-					     G_FILE_QUERY_INFO_NONE,
-					     G_PRIORITY_DEFAULT,
-					     NULL,
-					     (GAsyncReadyCallback) set_attributes_cb,
-					     NULL);
+		GError *error = NULL;
+
+		/* We save synchronously since metadata is always local so it
+		 * should be fast. Moreover this function can be called on
+		 * application shutdown, when the main loop has already exited,
+		 * so an async operation would not terminate.
+		 * https://bugzilla.gnome.org/show_bug.cgi?id=736591
+		 */
+		g_file_set_attributes_from_info (location,
+						 info,
+						 G_FILE_QUERY_INFO_NONE,
+						 NULL,
+						 &error);
+
+		if (error != NULL)
+		{
+			g_warning ("Set document metadata failed: %s", error->message);
+			g_error_free (error);
+		}
 	}
 
 	g_object_unref (info);
